package org.cyrano.editor.parts;

import java.util.HashMap;
import java.util.Map;

public class PartsRegister {

  private Map<Object, EditPart> editPartByModel = new HashMap<Object, EditPart>();

  private Map<EditPart, Object> modelByEditPart = new HashMap<EditPart, Object>();

  // --------------------------------------------------------------------------------

  private Map<Object, ViewPart> viewPartByModel = new HashMap<Object, ViewPart>();

  private Map<ViewPart, Object> modelByViewPart = new HashMap<ViewPart, Object>();

  // --------------------------------------------------------------------------------

  private PartsFactory partsFactory;

  // --------------------------------------------------------------------------------

  public PartsRegister(PartsFactory partsFactory) {
    this.partsFactory = partsFactory;
  }

  // --------------------------------------------------------------------------------

  public void bind(Object model) {
    partsFactory.bindPartsFor(model, this);
  }

  // --------------------------------------------------------------------------------

  public void bind(Object model, EditPart editPart, ViewPart viewPart) {
    if (editPartByModel.get(model) != null) {
      throw new IllegalStateException("EditPart for: " + model.toString());
    }

    if (viewPartByModel.get(model) != null) {
      throw new IllegalStateException("ViewPart for: " + model.toString());
    }

    editPartByModel.put(model, editPart);
    modelByEditPart.put(editPart, model);

    viewPartByModel.put(model, viewPart);
    modelByViewPart.put(viewPart, model);
  }

  // --------------------------------------------------------------------------------

  public void unbind(Object model) {
    modelByEditPart.remove(editPartByModel.remove(model));
    modelByViewPart.remove(viewPartByModel.remove(model));
  }

  // --------------------------------------------------------------------------------

  public void unbindAll() {
    modelByEditPart.clear();
    modelByViewPart.clear();
    editPartByModel.clear();
    viewPartByModel.clear();
  }

  // --------------------------------------------------------------------------------

  public EditPart getEditPartByModel(Object model) {
    return editPartByModel.get(model);
  }

  public ViewPart getViewPartByModel(Object model) {
    return viewPartByModel.get(model);
  }

  // --------------------------------------------------------------------------------

  public Object getModelByEditPart(EditPart editPart) {
    return modelByEditPart.get(editPart);
  }

  public Object getModelByViewPart(ViewPart viewPart) {
    return modelByViewPart.get(viewPart);
  }

  // --------------------------------------------------------------------------------

  public ViewPart getViewPartByEditPart(EditPart editPart) {
    return viewPartByModel.get(modelByEditPart.get(editPart));
  }

  public EditPart getEditPartByViewPart(ViewPart viewPart) {
    return editPartByModel.get(modelByViewPart.get(viewPart));
  }
}
