package org.cyrano.timetable.ui.command;

import java.util.ArrayList;
import java.util.List;

import org.cyrano.editor.listeners.Command;

public class ComposedCommand implements Command {

  public enum CanExecuteMode {
    ANY, ALL
  }

  // --------------------------------------------------------------------------------

  List<Command> commandList = new ArrayList<Command>();

  private final CanExecuteMode canExecuteMode;

  // --------------------------------------------------------------------------------

  public ComposedCommand(CanExecuteMode canExecuteMode) {
    this.canExecuteMode = canExecuteMode;
  }

  // --------------------------------------------------------------------------------

  public void add(Command command) {
    commandList.add(command);
  }

  // --------------------------------------------------------------------------------

  @Override
  public void redoCommand() {
    for (Command command : commandList) {
      if (command.canExecute()) {
        command.redoCommand();
      }
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  public void undoCommand() {
    for (Command command : commandList) {
      if (command.canExecute()) {
        command.undoCommand();
      }
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  public boolean canExecute() {
    boolean ret;

    if (canExecuteMode == CanExecuteMode.ANY) {
      ret = false;
    } else {
      ret = true;
    }

    for (Command command : commandList) {
      if (canExecuteMode == CanExecuteMode.ANY) {
        ret |= command.canExecute();
      } else {
        ret &= command.canExecute();
      }
    }

    return ret;
  }
}
