package org.cyrano.timetable.ui.edit;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.cyrano.editor.listeners.SelectionManager;
import org.cyrano.timetable.ui.model.base.BaseModel;
import org.cyrano.timetable.ui.model.base.ModelChangeEvent;
import org.cyrano.timetable.ui.model.base.ModelChangeListener;
import org.cyrano.timetable.ui.model.base.Parent;
import org.cyrano.timetable.ui.model.base.StructureChangeEvent;
import org.cyrano.timetable.ui.model.base.StructureChangeListener;
import org.cyrano.timetable.ui.model.timetable.DisplayMode;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.timetable.ui.model.visitor.ModelProcessor;
import org.cyrano.utils.ImageCache;

import com.jidesoft.swing.CheckBoxTree;

public class BaseTreePanel extends JPanel //
    implements
      StructureChangeListener,
      ModelChangeListener,
      DisplayModeProvider {

  // --------------------------------------------------------------------------------

  protected Map<BaseModel, RootTreeNode> nodesByModel = //
  new HashMap<BaseModel, RootTreeNode>();

  private DisplayMode displayMode;

  protected CheckBoxTree tree;

  protected Root root;

  // --------------------------------------------------------------------------------

  public BaseTreePanel(DisplayMode displayMode) {
    this.displayMode = displayMode;

    setLayout(new BorderLayout());

    initActions();

    add(initTopPanel(), BorderLayout.NORTH);

    tree = new CheckBoxTree();

    JScrollPane scrollPane = new JScrollPane();
    scrollPane.setViewportView(tree);
    add(scrollPane, BorderLayout.CENTER);

    tree.getSelectionModel().setSelectionMode( //
        TreeSelectionModel.SINGLE_TREE_SELECTION);

    tree.setCellRenderer(new ColorTreeCellRenderer());

    tree.addMouseListener(new MouseAdapter() {
      public void mousePressed/* */(MouseEvent evt) {
        BaseTreePanel.this.mousePressed/* */(evt);
      }

      public void mouseReleased/**/(MouseEvent evt) {
        BaseTreePanel.this.mouseReleased/**/(evt);
      }
    });

    tree.getSelectionModel(). //
        addTreeSelectionListener(new TreeSelectionListener() {
          @Override
          public void valueChanged(TreeSelectionEvent evt) {
            selectedValueChanged(evt);
          }
        });

    tree.getCheckBoxTreeSelectionModel(). //
        addTreeSelectionListener(new TreeSelectionListener() {
          @Override
          public void valueChanged(TreeSelectionEvent evt) {
            checkBoxValueChanged(evt);
          }
        });
  }

  // --------------------------------------------------------------------------------

  private JPanel initTopPanel() {
    JPanel ret = new JPanel(new FlowLayout(FlowLayout.RIGHT, 5, 2));

    JButton btnAdd = new JButton( //
        new ImageIcon(ImageCache.getInstance().getImage("images/16x16_create.png")));
    btnAdd.setMargin(new Insets(0, 0, 0, 0));
    btnAdd.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        btnCreateClicked();
      }
    });
    ret.add(btnAdd);

    JButton btnEdt = new JButton( //
        new ImageIcon(ImageCache.getInstance().getImage("images/16x16_update.png")));
    btnEdt.setMargin(new Insets(0, 0, 0, 0));
    btnEdt.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        btnUpdateClicked();
      }
    });
    ret.add(btnEdt);

    JButton btnDel = new JButton( //
        new ImageIcon(ImageCache.getInstance().getImage("images/16x16_delete.png")));
    btnDel.setMargin(new Insets(0, 0, 0, 0));
    btnDel.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        btnDeleteClicked();
      }
    });
    ret.add(btnDel);

    return ret;
  }

  // --------------------------------------------------------------------------------

  protected void btnCreateClicked() {
    // Empty
  }

  protected void btnUpdateClicked() {
    // Empty
  }

  protected void btnDeleteClicked() {
    // Empty
  }

  // --------------------------------------------------------------------------------

  public void setRoot(Root root) {

    if (this.root != null) {
      this.root.getStructureChangeSupport(). //
          delStructureChangeListener(this);
      this.root.getModelChangeSupport(). //
          delModelChangeListener(this);
    }

    this.root = root;

    this.root.getStructureChangeSupport(). //
        addStructureChangeListener(this);
    this.root.getModelChangeSupport(). //
        addModelChangeListener(this);

    RootTreeNode rootTreeNode = buildRootTreeNode();

    tree.expandPath(new TreePath(rootTreeNode));

    tree.getCheckBoxTreeSelectionModel(). //
        addSelectionPath(new TreePath(rootTreeNode));
  }

  // --------------------------------------------------------------------------------

  private RootTreeNode buildRootTreeNode() {
    DefaultTreeModel dtm = (DefaultTreeModel) tree.getModel();

    TreeBuilderVisitor treeBuilderVisitor = new TreeBuilderVisitor( //
        dtm, nodesByModel, displayMode);
    ModelProcessor modelProcessor = new ModelProcessor( //
        treeBuilderVisitor);

    modelProcessor.visit(root);

    return (RootTreeNode) dtm.getRoot();
  }

  // --------------------------------------------------------------------------------

  private void mousePressed/* */(MouseEvent evt) {
    if (evt.isPopupTrigger()) {
      popupEvent(evt);
    }
  }

  // --------------------------------------------------------------------------------

  private void mouseReleased/**/(MouseEvent evt) {
    if (evt.isPopupTrigger()) {
      popupEvent(evt);
    }
  }

  // --------------------------------------------------------------------------------

  private void popupEvent(MouseEvent evt) {
    JTree tree = (JTree) evt.getSource();

    TreePath path = tree.getPathForLocation( //
        evt.getPoint().x, evt.getPoint().y);

    if (path == null) {
      return;
    }

    tree.setSelectionPath(path);

    RootTreeNode node = //
    (RootTreeNode) path.getLastPathComponent();

    JPopupMenu popupMenu = createPopupMenuFor(node.getUserObject());

    if (popupMenu != null) {
      popupMenu.show( //
          tree, evt.getPoint().x, evt.getPoint().y);
    }
  }

  // --------------------------------------------------------------------------------

  protected void initActions() {
    // Empty
  }

  // --------------------------------------------------------------------------------

  protected JPopupMenu createPopupMenuFor(Object object) {
    return null;
  }

  // --------------------------------------------------------------------------------

  private void selectedValueChanged(TreeSelectionEvent evt) {
    SelectionManager.getInstance().clear();

    BaseModel data = (BaseModel) getSelectedObject();

    if (data != null) {
      SelectionManager.getInstance().add(data);
    }
  }

  // --------------------------------------------------------------------------------

  private void checkBoxValueChanged(TreeSelectionEvent evt) {
    TreePath[] treePaths = evt.getPaths();

    for (TreePath treePath : treePaths) {
      RootTreeNode lastPathComponent = //
      (RootTreeNode) treePath.getLastPathComponent();

      BaseModel data = (BaseModel) lastPathComponent.getUserObject();

      if (evt.isAddedPath(treePath)) {
        data.setVisible(true);
      } else {
        data.setVisible(false);
      }
    }
  }

  // --------------------------------------------------------------------------------

  protected BaseModel getSelectedObject() {
    TreePath treePath = tree.getSelectionPath();

    if (treePath == null) {
      return null;
    }

    RootTreeNode lastPathComponent = //
    (RootTreeNode) treePath.getLastPathComponent();

    return (BaseModel) lastPathComponent.getUserObject();
  }

  // --------------------------------------------------------------------------------

  @Override
  public void addChild(StructureChangeEvent evt) {
    RootTreeNode newChildTreeNode = new RootTreeNode(evt.getChild());

    DefaultTreeModel dtm = (DefaultTreeModel) tree.getModel();
    TreeBuilderVisitor treeBuilderVisitor = new TreeBuilderVisitor( //
        dtm, nodesByModel, displayMode);
    ModelProcessor modelProcessor = new ModelProcessor( //
        treeBuilderVisitor);
    modelProcessor.visit(newChildTreeNode.getUserObject());
  }

  // --------------------------------------------------------------------------------

  @Override
  public void delChild(StructureChangeEvent evt) {
    RootTreeNode childTreeNode = nodesByModel.get(evt.getChild());

    if (childTreeNode == null) {
      return;
    }

    DefaultTreeModel dtm = (DefaultTreeModel) tree.getModel();
    dtm.removeNodeFromParent(childTreeNode);
    delNodesRec(childTreeNode);
  }

  // --------------------------------------------------------------------------------

  @SuppressWarnings("unchecked")
  private void delNodesRec(RootTreeNode parTreeNode) {
    nodesByModel.remove(parTreeNode);

    Enumeration<RootTreeNode> enumeration = //
    (Enumeration<RootTreeNode>) parTreeNode.children();

    while (enumeration.hasMoreElements()) {
      RootTreeNode chlTreeNode = //
      (RootTreeNode) enumeration.nextElement();
      delNodesRec(chlTreeNode);
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  @SuppressWarnings({"unchecked", "rawtypes"})
  public void reparent(StructureChangeEvent evt) {
    RootTreeNode newParentTreeNode = nodesByModel.get(evt.getNewParent());
    RootTreeNode objectTreeNode = nodesByModel.get(evt.getChild());

    DefaultTreeModel dtm = (DefaultTreeModel) tree.getModel();
    dtm.removeNodeFromParent(objectTreeNode);

    dtm.insertNodeInto( //
        objectTreeNode, newParentTreeNode, //
        ((Parent) evt.getNewParent()).indexOf(evt.getChild()));

    TreePath treePath = new TreePath(dtm.getPathToRoot(newParentTreeNode));
    tree.expandPath(treePath);
  }

  // --------------------------------------------------------------------------------

  @Override
  public void modelChanged(ModelChangeEvent evt) {
    RootTreeNode childTreeNode = nodesByModel.get(evt.getModel());
    DefaultTreeModel dtm = (DefaultTreeModel) tree.getModel();
    dtm.nodeChanged(childTreeNode);
  }

  // --------------------------------------------------------------------------------

  @Override
  public DisplayMode getDisplayMode() {
    return displayMode;
  }

  @Override
  public void updateSelectedItems() {
    selectedValueChanged(null);
  }
}
