package org.cyrano.timetable.ui.edit;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JPanel;

public class JColorButton extends JButton {

  private static final Dimension COLOR_BUTTON_DIMENSION = //
  new Dimension(40, 15);

  private Color color;

  private JPanel lblColor;

  // --------------------------------------------------------------------------------

  public JColorButton() {
    this(Color.BLACK);
  }

  // --------------------------------------------------------------------------------

  public JColorButton(Color color) {
    this.color = color;

    initGUI();

    addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent evt) {
        selectColor();
      }
    });
  }

  // --------------------------------------------------------------------------------

  private void selectColor() {
    Color selectedColor = JColorChooser.showDialog(this, "", color);

    if (selectedColor != null) {
      color = selectedColor;
      lblColor.setBackground(color);
    }
  }

  // --------------------------------------------------------------------------------

  private void initGUI() {
    setMargin(new Insets(3, 3, 3, 3));

    lblColor = new JPanel();
    lblColor.setPreferredSize(COLOR_BUTTON_DIMENSION);
    lblColor.setBackground(color);
    lblColor.setOpaque(true);
    add(lblColor);
  }

  // --------------------------------------------------------------------------------

  public Color getColor() {
    return color;
  }

  // --------------------------------------------------------------------------------

  public void setColor(Color color) {
    this.color = color;
    lblColor.setBackground(color);
  }
}
