package org.cyrano.timetable.ui.edit;

import java.util.Map;

import javax.swing.tree.DefaultTreeModel;

import org.cyrano.timetable.ui.model.base.BaseModel;
import org.cyrano.timetable.ui.model.timetable.Cour;
import org.cyrano.timetable.ui.model.timetable.DisplayMode;
import org.cyrano.timetable.ui.model.timetable.Prof;
import org.cyrano.timetable.ui.model.timetable.Room;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.timetable.ui.model.timetable.Sect;
import org.cyrano.timetable.ui.model.timetable.Sems;
import org.cyrano.timetable.ui.model.timetable.Slot;
import org.cyrano.timetable.ui.model.visitor.ModelVisitor;

public class TreeBuilderVisitor implements ModelVisitor {

  private Map<BaseModel, RootTreeNode> nodesByModel;

  private DefaultTreeModel defaultTreeModel;

  private DisplayMode displayMode;

  // --------------------------------------------------------------------------------

  public TreeBuilderVisitor( //
      DefaultTreeModel defaultTreeModel, //
      Map<BaseModel, RootTreeNode> nodesByModel, //
      DisplayMode displayMode) {

    this.defaultTreeModel = defaultTreeModel;
    this.nodesByModel/* */= nodesByModel;
    this.displayMode/*  */= displayMode;
  }

  // --------------------------------------------------------------------------------
  // TimeTableVisitor
  // --------------------------------------------------------------------------------

  @Override
  public void visitRoot(Root timeTable) {
    RootTreeNode tttn = //
    new RootTreeNode(timeTable);

    defaultTreeModel.setRoot(tttn);
    nodesByModel.put(timeTable, tttn);
  }

  // --------------------------------------------------------------------------------

  @Override
  public void visitSems(Sems sems) {
    if (displayMode != DisplayMode.SEMS) {
      return;
    }

    RootTreeNode tttn = //
    new RootTreeNode(sems);

    nodesByModel.put(sems, tttn);

    RootTreeNode parent = //
    nodesByModel.get(sems.getRoot());

    defaultTreeModel.insertNodeInto(tttn, parent, //
        sems.getRoot().getSemsList().indexOf(sems));
  }

  // --------------------------------------------------------------------------------

  @Override
  public void visitProf(Prof prof) {
    if (displayMode != DisplayMode.PROF) {
      return;
    }

    RootTreeNode tttn = //
    new RootTreeNode(prof);

    nodesByModel.put(prof, tttn);

    RootTreeNode parent = //
    nodesByModel.get(prof.getRoot());

    defaultTreeModel.insertNodeInto(tttn, parent, //
        prof.getRoot().getProfList().indexOf(prof));
  }

  // --------------------------------------------------------------------------------

  @Override
  public void visitRoom(Room room) {
    if (displayMode != DisplayMode.ROOM) {
      return;
    }

    RootTreeNode tttn = //
    new RootTreeNode(room);

    nodesByModel.put(room, tttn);

    RootTreeNode parent = //
    nodesByModel.get(room.getRoot());

    defaultTreeModel.insertNodeInto(tttn, parent, //
        room.getRoot().getRoomList().indexOf(room));
  }

  // --------------------------------------------------------------------------------

  @Override
  public void visitCour(Cour cour) {
    if (displayMode != DisplayMode.SEMS) {
      return;
    }

    RootTreeNode tttn = //
    new RootTreeNode(cour);

    nodesByModel.put(cour, tttn);

    RootTreeNode parent = //
    nodesByModel.get(cour.getSemsRef().getParentRef());

    defaultTreeModel.insertNodeInto(tttn, parent, //
        cour.getSemsRef().getParentRef().getCourList().indexOf(cour));
  }

  // --------------------------------------------------------------------------------

  @Override
  public void visitSect(Sect sect) {
    if (displayMode != DisplayMode.SEMS) {
      return;
    }

    RootTreeNode tttn = //
    new RootTreeNode(sect);

    nodesByModel.put(sect, tttn);

    RootTreeNode parent = //
    nodesByModel.get(sect.getCourRef().getParentRef());

    defaultTreeModel.insertNodeInto(tttn, parent, //
        sect.getCourRef().getParentRef().getSectList().indexOf(sect));
  }

  // --------------------------------------------------------------------------------

  @Override
  public void visitSlot(Slot slot) {
    // Empty
  }
}
