package org.cyrano.timetable.ui.edit.prof;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JPopupMenu;

import org.cyrano.editor.listeners.UndoRedoManager;
import org.cyrano.timetable.ui.command.ClearProfFromSectCommand;
import org.cyrano.timetable.ui.command.ComposedCommand;
import org.cyrano.timetable.ui.command.ComposedCommand.CanExecuteMode;
import org.cyrano.timetable.ui.command.DeleteChildCommand;
import org.cyrano.timetable.ui.edit.BaseTreePanel;
import org.cyrano.timetable.ui.main.MainFrame;
import org.cyrano.timetable.ui.model.base.BaseModel;
import org.cyrano.timetable.ui.model.timetable.DisplayMode;
import org.cyrano.timetable.ui.model.timetable.Prof;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.utils.ImageCache;

public class ProfTreePanel extends BaseTreePanel {

  private AbstractAction actCreateProf;
  private AbstractAction actUpdateProf;
  private AbstractAction actDeleteProf;

  // --------------------------------------------------------------------------------

  public ProfTreePanel() {
    super(DisplayMode.PROF);
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void initActions() {
    actCreateProf = new AbstractAction(//
        _I18NProfTreePanel.createProf(), //
        new ImageIcon(ImageCache.getInstance().getImage( //
            "images/16x16_create.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        actCreateProfClicked();
      }
    };
    actUpdateProf = new AbstractAction(//
        _I18NProfTreePanel.updateProf(), //
        new ImageIcon(ImageCache.getInstance().getImage( //
            "images/16x16_update.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        actUpdateProfClicked();
      }
    };
    actDeleteProf = new AbstractAction(//
        _I18NProfTreePanel.deleteProf(), //
        new ImageIcon(ImageCache.getInstance().getImage( //
            "images/16x16_delete.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        actDeleteProfClicked();
      }
    };
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void btnCreateClicked() {
    BaseModel baseModel = getSelectedObject();

    if (baseModel instanceof Root) {
      actCreateProf.actionPerformed(null);
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void btnUpdateClicked() {
    BaseModel baseModel = getSelectedObject();

    if (baseModel instanceof Prof) {
      actUpdateProf.actionPerformed(null);
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void btnDeleteClicked() {
    BaseModel baseModel = getSelectedObject();

    if (baseModel instanceof Prof) {
      actDeleteProf.actionPerformed(null);
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  protected JPopupMenu createPopupMenuFor(Object object) {
    if (object instanceof Root) {
      return createPopupMenuForRoot();
    }

    if (object instanceof Prof) {
      return createPopupMenuForProf();
    }

    return null;
  }

  // --------------------------------------------------------------------------------

  private JPopupMenu createPopupMenuForRoot() {
    JPopupMenu ret = new JPopupMenu();
    ret.add(actCreateProf);
    return ret;
  }

  private JPopupMenu createPopupMenuForProf() {
    JPopupMenu ret = new JPopupMenu();
    ret.add(actUpdateProf);
    ret.add(actDeleteProf);
    return ret;
  }

  // --------------------------------------------------------------------------------

  private void actCreateProfClicked() {
    Root root = (Root) getSelectedObject();
    DlgEditProf edt = new DlgEditProf( //
        MainFrame.workbenchRef, null, root);
    edt.setVisible(true);
  }

  // --------------------------------------------------------------------------------

  protected void actUpdateProfClicked() {
    Prof prof = (Prof) getSelectedObject();
    DlgEditProf edt = new DlgEditProf( //
        MainFrame.workbenchRef, prof, null);
    edt.setVisible(true);
  }

  // --------------------------------------------------------------------------------

  protected void actDeleteProfClicked() {
    Prof prof = (Prof) getSelectedObject();

    ClearProfFromSectCommand clearProfCommand = //
    new ClearProfFromSectCommand(prof);

    DeleteChildCommand<Root, Prof> deleteCommand = //
    new DeleteChildCommand<Root, Prof>( //
        prof.getRootRef().getParentRef().getProfList(), prof);

    ComposedCommand composedCommand = //
    new ComposedCommand(CanExecuteMode.ANY);

    composedCommand.add(deleteCommand);
    composedCommand.add(clearProfCommand);

    UndoRedoManager.getInstance().execute(composedCommand);
  }
}
