package org.cyrano.timetable.ui.edit.room;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JPopupMenu;

import org.cyrano.editor.listeners.UndoRedoManager;
import org.cyrano.timetable.ui.command.ClearRoomFromSlotCommand;
import org.cyrano.timetable.ui.command.ComposedCommand;
import org.cyrano.timetable.ui.command.ComposedCommand.CanExecuteMode;
import org.cyrano.timetable.ui.command.DeleteChildCommand;
import org.cyrano.timetable.ui.edit.BaseTreePanel;
import org.cyrano.timetable.ui.main.MainFrame;
import org.cyrano.timetable.ui.model.base.BaseModel;
import org.cyrano.timetable.ui.model.timetable.DisplayMode;
import org.cyrano.timetable.ui.model.timetable.Room;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.utils.ImageCache;

public class RoomTreePanel extends BaseTreePanel {

  private AbstractAction actCreateRoom;
  private AbstractAction actUpdateRoom;
  private AbstractAction actDeleteRoom;

  // --------------------------------------------------------------------------------

  public RoomTreePanel() {
    super(DisplayMode.ROOM);
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void initActions() {
    actCreateRoom = new AbstractAction(//
        _I18NRoomTreePanel.createRoom(), //
        new ImageIcon(ImageCache.getInstance().getImage( //
            "images/16x16_create.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        actCreateRoomClicked();
      }
    };
    actUpdateRoom = new AbstractAction(//
        _I18NRoomTreePanel.updateRoom(), //
        new ImageIcon(ImageCache.getInstance().getImage( //
            "images/16x16_update.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        actUpdateRoomClicked();
      }
    };
    actDeleteRoom = new AbstractAction(//
        _I18NRoomTreePanel.deleteRoom(), //
        new ImageIcon(ImageCache.getInstance().getImage( //
            "images/16x16_delete.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        actDeleteRoomClicked();
      }
    };
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void btnCreateClicked() {
    BaseModel baseModel = getSelectedObject();

    if (baseModel instanceof Root) {
      actCreateRoom.actionPerformed(null);
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void btnUpdateClicked() {
    BaseModel baseModel = getSelectedObject();

    if (baseModel instanceof Room) {
      actUpdateRoom.actionPerformed(null);
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void btnDeleteClicked() {
    BaseModel baseModel = getSelectedObject();

    if (baseModel instanceof Room) {
      actDeleteRoom.actionPerformed(null);
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  protected JPopupMenu createPopupMenuFor(Object object) {
    if (object instanceof Root) {
      return createPopupMenuForTimeTable();
    }

    if (object instanceof Room) {
      return createPopupMenuForRoom();
    }

    return null;
  }

  // --------------------------------------------------------------------------------

  private JPopupMenu createPopupMenuForTimeTable() {
    JPopupMenu ret = new JPopupMenu();
    ret.add(actCreateRoom);
    return ret;
  }

  private JPopupMenu createPopupMenuForRoom() {
    JPopupMenu ret = new JPopupMenu();
    ret.add(actUpdateRoom);
    ret.add(actDeleteRoom);
    return ret;
  }

  // --------------------------------------------------------------------------------

  private void actCreateRoomClicked() {
    Root root = (Root) getSelectedObject();
    DlgEditRoom edt = new DlgEditRoom( //
        MainFrame.workbenchRef, null, root);
    edt.setVisible(true);
  }

  // --------------------------------------------------------------------------------

  protected void actUpdateRoomClicked() {
    Room prof = (Room) getSelectedObject();
    DlgEditRoom edt = new DlgEditRoom( //
        MainFrame.workbenchRef, prof, null);
    edt.setVisible(true);
  }

  // --------------------------------------------------------------------------------

  protected void actDeleteRoomClicked() {
    Room room = (Room) getSelectedObject();

    ClearRoomFromSlotCommand clearRoomCommand = //
    new ClearRoomFromSlotCommand(room);

    DeleteChildCommand<Root, Room> deleteCommand = //
    new DeleteChildCommand<Root, Room>( //
        room.getRootRef().getParentRef().getRoomList(), room);

    ComposedCommand composedCommand = //
    new ComposedCommand(CanExecuteMode.ANY);

    composedCommand.add(deleteCommand);
    composedCommand.add(clearRoomCommand);

    UndoRedoManager.getInstance().execute(composedCommand);
  }
}
