package org.cyrano.timetable.ui.edit.sems;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.commons.lang.StringUtils;
import org.cyrano.editor.listeners.Command;
import org.cyrano.editor.listeners.UndoRedoManager;
import org.cyrano.timetable.ui.command.CreateChildCommand;
import org.cyrano.timetable.ui.command.UpdateChildCommand;
import org.cyrano.timetable.ui.edit.DlgEditBase;
import org.cyrano.timetable.ui.edit.JColorButton;
import org.cyrano.timetable.ui.main.MainFrame;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.timetable.ui.model.timetable.Sems;

public class DlgEditSems extends DlgEditBase<Sems> {

  private Root root;

  private JTextField txtName;

  private JColorButton btnBackground;

  // --------------------------------------------------------------------------------

  public DlgEditSems(JFrame parent, Sems data, Root root) {
    super(parent, data, //
        _I18NDlgEditSems.createTitle(), //
        _I18NDlgEditSems.updateTitle());

    this.root = root;

    guiFromData(data);
  }

  // --------------------------------------------------------------------------------

  private void dataFromGui(Sems sems) {
    sems.setName(txtName.getText());
    sems.setBackground(btnBackground.getColor());
  }

  // --------------------------------------------------------------------------------

  private void guiFromData(Sems sems) {
    if (sems == null) {
      return;
    }

    txtName.setText(sems.getName());
    btnBackground.setColor(sems.getBackground());
  }

  // --------------------------------------------------------------------------------

  @Override
  protected JPanel initUpdatePanel() {
    JPanel ret = new JPanel(new GridBagLayout());

    ret.setBorder(BorderFactory.createTitledBorder(//
        _I18NDlgEditSems.titledBorder()));

    int row = 0;

    GridBagConstraints gbc;

    // ----------------------------------------

    JLabel lblName = new JLabel(_I18NDlgEditSems.name());
    gbc = initGBC(0, row, 0, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 0);
    ret.add(lblName, gbc);

    txtName = new JTextField();
    txtName.setPreferredSize(new Dimension(200, 25));
    gbc = initGBC(1, row, 1, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 5);
    ret.add(txtName, gbc);

    // ----------------------------------------

    row++;

    JLabel lblBackground = new JLabel(_I18NDlgEditSems.background());
    gbc = initGBC(0, row, 0, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 0);
    ret.add(lblBackground, gbc);

    btnBackground = new JColorButton();
    gbc = initGBC(1, row, 0, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.NONE;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 5);
    ret.add(btnBackground, gbc);

    return ret;
  }

  // --------------------------------------------------------------------------------

  @Override
  protected boolean validateData() {
    if (StringUtils.isBlank(txtName.getText())) {
      JOptionPane.showMessageDialog(MainFrame.workbenchRef, //
          _I18NDlgEditSems.errorEmptyName(), //
          _I18NDlgEditSems.errorTitle(), //
          JOptionPane.ERROR_MESSAGE);

      return false;
    }

    return true;
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void createBtnAcceptClicked() {
    data = new Sems();
    dataFromGui(data);

    Command command = //
    new CreateChildCommand<Root, Sems>( //
        root.getSemsList(), data);

    UndoRedoManager.getInstance().execute(command);

    dispose();
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void updateBtnAcceptClicked() {
    Sems prevSems = new Sems();
    prevSems.shallowDataCopyFrom(data);

    Sems nextSems = new Sems();
    dataFromGui(nextSems);

    Command command = new UpdateChildCommand( //
        prevSems, data, nextSems);

    UndoRedoManager.getInstance().execute(command);

    dispose();
  }
}
