package org.cyrano.timetable.ui.edit.slot;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.cyrano.editor.listeners.Command;
import org.cyrano.editor.listeners.UndoRedoManager;
import org.cyrano.timetable.ui.command.UpdateSlotCommand;
import org.cyrano.timetable.ui.edit.DlgEditBase;
import org.cyrano.timetable.ui.edit.LabelGetter;
import org.cyrano.timetable.ui.edit.LabelValueBeanComboBoxModel;
import org.cyrano.timetable.ui.model.timetable.Room;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.timetable.ui.model.timetable.Sect;
import org.cyrano.timetable.ui.model.timetable.Slot;

import com.minotauro.cleda.util.LabelValueBean;

public class DlgUpdateSlot extends DlgEditBase<Slot> {

  private Sect sect;

  private JComboBox<LabelValueBean<Room>> cboRoom;

  // --------------------------------------------------------------------------------

  public DlgUpdateSlot(JFrame parent, Slot data, Sect sect) {
    super(parent, data, //
        _I18NDlgEditSlot.createTitle(), //
        _I18NDlgEditSlot.updateTitle());

    this.sect = sect;

    guiFromData(data);
    initCboRoom();
  }

  // --------------------------------------------------------------------------------

  private void dataFromGui(Slot slot) {
    LabelValueBeanComboBoxModel<Room> dcbm = //
    (LabelValueBeanComboBoxModel<Room>) cboRoom.getModel();

    slot.setRoom(dcbm.getSelectedData());
  }

  // --------------------------------------------------------------------------------

  private void guiFromData(Slot slot) {
    if (slot == null) {
      return;
    }

    LabelValueBeanComboBoxModel<Room> dcbm = //
    (LabelValueBeanComboBoxModel<Room>) cboRoom.getModel();

    dcbm.setSelectedData(slot.getRoom());
  }

  // --------------------------------------------------------------------------------

  @Override
  protected JPanel initUpdatePanel() {
    JPanel ret = new JPanel(new GridBagLayout());

    ret.setBorder(BorderFactory.createTitledBorder(//
        _I18NDlgEditSlot.titledBorder()));

    int row = 0;

    GridBagConstraints gbc;

    JLabel lblRoom = new JLabel(_I18NDlgEditSlot.room());
    gbc = initGBC(0, row, 0, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 0);
    ret.add(lblRoom, gbc);

    cboRoom = new JComboBox<LabelValueBean<Room>>();
    cboRoom.setModel(new LabelValueBeanComboBoxModel<Room>());
    cboRoom.setPreferredSize(new Dimension(200, 25));
    gbc = initGBC(1, row, 1, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 5);
    ret.add(cboRoom, gbc);

    return ret;
  }

  // --------------------------------------------------------------------------------

  private void initCboRoom() {
    Root root = data != null ? data.getRoot() : sect.getRoot();

    LabelValueBeanComboBoxModel<Room> dcbm = //
    (LabelValueBeanComboBoxModel<Room>) cboRoom.getModel();
    dcbm.removeAllElements();

    dcbm.add(_I18NDlgEditSlot.select(), null);

    dcbm.addAll(root.getRoomList().getChildrenList(), //
        new LabelGetter<Room>() {
          @Override
          public String getLabel(Room data) {
            return data.getName();
          }
        });

    if (data != null && data.getRoom() != null) {
      dcbm.setSelectedData(data.getRoom());
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void createBtnAcceptClicked() {
    throw new UnsupportedOperationException();
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void updateBtnAcceptClicked() {
    Slot prevSlot = new Slot();
    prevSlot.shallowDataCopyFrom(data);

    Slot nextSlot = new Slot();
    dataFromGui(nextSlot);

    Command command = new UpdateSlotCommand( //
        prevSlot, data, nextSlot);

    UndoRedoManager.getInstance().execute(command);

    dispose();
  }
}
