package org.cyrano.timetable.ui.figures;

import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.util.EventObject;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JPopupMenu;

import org.cyrano.editor.listeners.Command;
import org.cyrano.editor.listeners.UndoRedoManager;
import org.cyrano.editor.parts.BaseEditPart;
import org.cyrano.editor.parts.PartsRegister;
import org.cyrano.editor.tool.EditPolicyExecutor;
import org.cyrano.editor.tool.EditPolicyMouseHandler;
import org.cyrano.editor.tool.RectangleAddFeedback;
import org.cyrano.editor.tool.RectangleHFeedback;
import org.cyrano.editor.tool.RectangleMouseHandler;
import org.cyrano.editor.tool.RectangleXYFeedback;
import org.cyrano.editor.tool.Tool;
import org.cyrano.timetable.ui.command.DeleteChildCommand;
import org.cyrano.timetable.ui.command.SlotWHCommand;
import org.cyrano.timetable.ui.command.SlotXYCommand;
import org.cyrano.timetable.ui.edit.slot.DlgCreateSlot;
import org.cyrano.timetable.ui.edit.slot.DlgUpdateSlot;
import org.cyrano.timetable.ui.main.MainFrame;
import org.cyrano.timetable.ui.model.hour.Hour;
import org.cyrano.timetable.ui.model.hour.HourModel;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.timetable.ui.model.timetable.Sect;
import org.cyrano.timetable.ui.model.timetable.Slot;
import org.cyrano.timetable.ui.tool.AddTool;
import org.cyrano.timetable.ui.tool.PointerTool;
import org.cyrano.utils.ImageCache;

public class SlotEditPart extends BaseEditPart {

  public SlotEditPart(PartsRegister partsRegister) {
    super(partsRegister);
  }

  // --------------------------------------------------------------------------------

  @Override
  public EditPolicyMouseHandler getEditPolicy(Tool tool, EventObject evt) {

    // --------------------------------------------------------------------------------
    // TODO: Nasty!!!
    // --------------------------------------------------------------------------------

    if (tool instanceof AddTool) {

      Slot slot = //
      (Slot) getPartsRegister().getModelByEditPart(this);
      final Root timeTable = slot.getRoot();

      TimeTableViewPart timeTableViewPart = //
      (TimeTableViewPart) getPartsRegister().getViewPartByModel(timeTable);

      EditPolicyMouseHandler editPolicyMouseHandler = //
      new RectangleMouseHandler(timeTableViewPart);

      editPolicyMouseHandler.init(//
          new RectangleAddFeedback(), new EditPolicyExecutor() {

            @Override
            public Command execute(EditPolicyMouseHandler editPolicyMouseHandler) {

              HourModel hourModel = timeTable.getHourModel();

              RectangleMouseHandler rmh = (RectangleMouseHandler) editPolicyMouseHandler;

              int begX = rmh.getClickDwPt().x < rmh.getClickUpPt().x ? rmh.getClickDwPt().x : rmh.getClickUpPt().x;
              int begY = rmh.getClickDwPt().y < rmh.getClickUpPt().y ? rmh.getClickDwPt().y : rmh.getClickUpPt().y;

              int endX = rmh.getClickDwPt().x > rmh.getClickUpPt().x ? rmh.getClickDwPt().x : rmh.getClickUpPt().x;
              int endY = rmh.getClickDwPt().y > rmh.getClickUpPt().y ? rmh.getClickDwPt().y : rmh.getClickUpPt().y;

              Hour hourBeg = hourModel.getHourAt(begX, begY);
              Hour hourEnd = hourModel.getHourAt(endX, endY);

              DlgCreateSlot csd = new DlgCreateSlot(MainFrame.workbenchRef, timeTable, hourBeg.day, hourBeg.beg,
                  hourEnd.end);
              csd.setVisible(true);

              return new Command() {

                @Override
                public void undoCommand() {
                  // TODO Auto-generated method stub

                }

                @Override
                public void redoCommand() {
                  // TODO Auto-generated method stub

                }

                @Override
                public boolean canExecute() {
                  // TODO Auto-generated method stub
                  return false;
                } // --------------------------------------------------------------------------------

              };
            }
          });
      return editPolicyMouseHandler;

    }

    // --------------------------------------------------------------------------------

    if (!(tool instanceof PointerTool)) {
      return null;
    }

    if (!(evt instanceof MouseEvent)) {
      throw new IllegalArgumentException("!(evt instanceof MouseEvent)");
    }

    MouseEvent mouseEvent = (MouseEvent) evt;

    SlotViewPart slotViewPart = //
    (SlotViewPart) getPartsRegister().getViewPartByEditPart(this);

    EditPolicyMouseHandler editPolicyMouseHandler = //
    new RectangleMouseHandler(slotViewPart);

    if (slotViewPart.getHandleRect().contains( //
        mouseEvent.getPoint().x, mouseEvent.getPoint().y)) {
      initWHEditPolicy(editPolicyMouseHandler);
    } else {
      initXYEditPolicy(editPolicyMouseHandler);
    }

    return editPolicyMouseHandler;
  }

  // --------------------------------------------------------------------------------

  @Override
  public JPopupMenu getPopupMenu(Tool tool, EventObject evt) {
    if (!(tool instanceof PointerTool)) {
      return null;
    }

    if (!(evt instanceof MouseEvent)) {
      throw new IllegalArgumentException("!(evt instanceof MouseEvent)");
    }

    JPopupMenu ret = new JPopupMenu();

    ret.add(new AbstractAction(_I18NSlotEditPart.update(), //
        new ImageIcon(ImageCache.getInstance().getImage("images/16x16_update.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        updateActionClicked();
      }
    });

    ret.add(new AbstractAction(_I18NSlotEditPart.delete(), //
        new ImageIcon(ImageCache.getInstance().getImage("images/16x16_delete.png"))) {
      @Override
      public void actionPerformed(ActionEvent evt) {
        deleteActionClicked();
      }
    });

    return ret;
  }

  // --------------------------------------------------------------------------------

  private void updateActionClicked() {
    Slot slot = (Slot) getPartsRegister().getModelByEditPart(this);
    DlgUpdateSlot esd = new DlgUpdateSlot( //
        MainFrame.workbenchRef, slot, null);
    esd.setVisible(true);
  }

  // --------------------------------------------------------------------------------

  private void deleteActionClicked() {
    Slot slot = (Slot) getPartsRegister().getModelByEditPart(this);

    DeleteChildCommand<Sect, Slot> command = //
    new DeleteChildCommand<Sect, Slot>( //
        slot.getSectRef().getParentRef().getSlotList(), slot);

    UndoRedoManager.getInstance().execute(command);
  }

  // --------------------------------------------------------------------------------

  private void initWHEditPolicy(EditPolicyMouseHandler editPolicyMouseHandler) {
    editPolicyMouseHandler.init(//
        new RectangleHFeedback(), new EditPolicyExecutor() {

          @Override
          public Command execute(EditPolicyMouseHandler editPolicyMouseHandler) {

            RectangleMouseHandler rectangleMouseHandler = //
            (RectangleMouseHandler) editPolicyMouseHandler;

            return new SlotWHCommand(rectangleMouseHandler, //
                (Slot) getPartsRegister().getModelByEditPart(SlotEditPart.this));
          }
        });
  }

  // --------------------------------------------------------------------------------

  private void initXYEditPolicy(EditPolicyMouseHandler editPolicyMouseHandler) {
    editPolicyMouseHandler.init(//
        new RectangleXYFeedback(), new EditPolicyExecutor() {

          @Override
          public Command execute(EditPolicyMouseHandler editPolicyMouseHandler) {

            RectangleMouseHandler rectangleMouseHandler = //
            (RectangleMouseHandler) editPolicyMouseHandler;

            return new SlotXYCommand(rectangleMouseHandler, //
                (Slot) getPartsRegister().getModelByEditPart(SlotEditPart.this));
          }
        });
  }
}
