package org.cyrano.timetable.ui.figures;

import java.awt.event.MouseEvent;
import java.util.EventObject;

import javax.swing.JPopupMenu;

import org.cyrano.editor.listeners.Command;
import org.cyrano.editor.parts.BaseContainerEditPart;
import org.cyrano.editor.parts.PartsRegister;
import org.cyrano.editor.tool.EditPolicyExecutor;
import org.cyrano.editor.tool.EditPolicyMouseHandler;
import org.cyrano.editor.tool.RectangleAddFeedback;
import org.cyrano.editor.tool.RectangleMouseHandler;
import org.cyrano.editor.tool.Tool;
import org.cyrano.timetable.ui.edit.slot.DlgCreateSlot;
import org.cyrano.timetable.ui.main.MainFrame;
import org.cyrano.timetable.ui.model.hour.Hour;
import org.cyrano.timetable.ui.model.hour.HourModel;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.timetable.ui.tool.AddTool;

public class TimeTableEditPart extends BaseContainerEditPart {

  public TimeTableEditPart(PartsRegister partsRegister) {
    super(partsRegister);
  }

  // --------------------------------------------------------------------------------

  @Override
  public EditPolicyMouseHandler getEditPolicy(Tool tool, EventObject evt) {
    if (!(tool instanceof AddTool)) {
      return null;
    }

    if (!(evt instanceof MouseEvent)) {
      throw new IllegalArgumentException("!(evt instanceof MouseEvent)");
    }

    //    MouseEvent mouseEvent = (MouseEvent) evt;

    final Root timeTable = //
    (Root) getPartsRegister().getModelByEditPart(this);

    TimeTableViewPart timeTableViewPart = //
    (TimeTableViewPart) getPartsRegister().getViewPartByEditPart(this);

    EditPolicyMouseHandler editPolicyMouseHandler = //
    new RectangleMouseHandler(timeTableViewPart);

    editPolicyMouseHandler.init(//
        new RectangleAddFeedback(), new EditPolicyExecutor() {

          @Override
          public Command execute(EditPolicyMouseHandler editPolicyMouseHandler) {

            HourModel hourModel = timeTable.getHourModel();

            RectangleMouseHandler rmh = (RectangleMouseHandler) editPolicyMouseHandler;

            int begX = rmh.getClickDwPt().x < rmh.getClickUpPt().x ? rmh
                .getClickDwPt().x : rmh.getClickUpPt().x;
            int begY = rmh.getClickDwPt().y < rmh.getClickUpPt().y ? rmh
                .getClickDwPt().y : rmh.getClickUpPt().y;

            int endX = rmh.getClickDwPt().x > rmh.getClickUpPt().x ? rmh
                .getClickDwPt().x : rmh.getClickUpPt().x;
            int endY = rmh.getClickDwPt().y > rmh.getClickUpPt().y ? rmh
                .getClickDwPt().y : rmh.getClickUpPt().y;

            Hour hourBeg = hourModel.getHourAt(begX, begY);
            Hour hourEnd = hourModel.getHourAt(endX, endY);

            DlgCreateSlot csd = new DlgCreateSlot(MainFrame.workbenchRef,
                timeTable, hourBeg.day, hourBeg.beg, hourEnd.end);
            csd.setVisible(true);

            return new Command() {

              @Override
              public void undoCommand() {
                // TODO Auto-generated method stub

              }

              @Override
              public void redoCommand() {
                // TODO Auto-generated method stub

              }

              @Override
              public boolean canExecute() {
                // TODO Auto-generated method stub
                return false;
              }
            };
          }
        });

    return editPolicyMouseHandler;
  }

  // --------------------------------------------------------------------------------

  @Override
  public JPopupMenu getPopupMenu(Tool tool, EventObject evt) {
    return null;
  }
}
