package org.cyrano.timetable.ui.model.base;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.cyrano.timetable.ui.model.base.StructureChangeListener.StructureChangeMethod;

public class ParentChildrenImpl<PARENT_TYPE, CHILDREN_TYPE>
    implements
      Parent<PARENT_TYPE, CHILDREN_TYPE>,
      Children<PARENT_TYPE> {

  // --------------------------------------------------------------------------------

  public final ObjectMatcher<CHILDREN_TYPE, String> ALL_MATCHER = //
  new ObjectMatcher<CHILDREN_TYPE, String>() {
    @Override
    public boolean matches(CHILDREN_TYPE object, String target) {
      return true;
    }
  };

  // --------------------------------------------------------------------------------

  private Setter<CHILDREN_TYPE, Children<PARENT_TYPE>> setter;

  private StructureChangeSupport structureChangeSupport;

  private Comparator<CHILDREN_TYPE> comparator;

  // --------------------------------------------------------------------------------

  private final List<CHILDREN_TYPE> childrenList = //
  new ArrayList<CHILDREN_TYPE>();

  private final PARENT_TYPE/*     */parentRef;

  // --------------------------------------------------------------------------------

  public ParentChildrenImpl(PARENT_TYPE parentRef) {
    this.parentRef = parentRef;
  }

  // --------------------------------------------------------------------------------

  @Override
  public void setSetter( //
      Setter<CHILDREN_TYPE, Children<PARENT_TYPE>> setter) {

    this.setter = setter;
  }

  // --------------------------------------------------------------------------------

  @Override
  public void setStructureChangeSupport(
      StructureChangeSupport structureChangeSupport) {

    this.structureChangeSupport = structureChangeSupport;
  }

  // --------------------------------------------------------------------------------

  @Override
  public void setComparator( //
      Comparator<CHILDREN_TYPE> comparator) {

    this.comparator = comparator;
  }

  // --------------------------------------------------------------------------------

  @Override
  public void add(CHILDREN_TYPE child, boolean fireEvent) {
    childrenList.add(child);

    if (comparator != null) {
      Collections.sort(childrenList, comparator);
    }

    if (setter != null) {
      setter.set(child, this);
    }

    if (fireEvent && structureChangeSupport != null) {
      structureChangeSupport.fireStructureChange( //
          new StructureChangeEvent(this, //
              child, (Parent<PARENT_TYPE, CHILDREN_TYPE>) this, null), //
          StructureChangeMethod.ADDCHILD);
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  public void add(CHILDREN_TYPE child) {
    add(child, true);
  }

  // --------------------------------------------------------------------------------

  @Override
  public void del(CHILDREN_TYPE child, boolean fireEvent) {
    childrenList.remove(child);

    if (setter != null) {
      setter.set(child, null);
    }

    if (fireEvent && structureChangeSupport != null) {
      structureChangeSupport.fireStructureChange( //
          new StructureChangeEvent(this, //
              child, (Parent<PARENT_TYPE, CHILDREN_TYPE>) this, null), //
          StructureChangeMethod.DELCHILD);
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  public void del(CHILDREN_TYPE child) {
    del(child, true);
  }

  // --------------------------------------------------------------------------------

  @Override
  public List<CHILDREN_TYPE> getChildrenList() {
    return new ArrayList<CHILDREN_TYPE>(childrenList);
  }

  // --------------------------------------------------------------------------------

  @Override
  public int indexOf(CHILDREN_TYPE child) {
    return childrenList.indexOf(child);
  }

  // --------------------------------------------------------------------------------

  @Override
  public PARENT_TYPE getParentRef() {
    return parentRef;
  }

  // --------------------------------------------------------------------------------

  @Override
  public void sort() {
    Collections.sort(childrenList, comparator);
  }

  // --------------------------------------------------------------------------------

  @Override
  public <TARGET> List<CHILDREN_TYPE>/**/findMultChildBy(
      ObjectMatcher<CHILDREN_TYPE, TARGET> objectMatcher, TARGET target) {

    List<CHILDREN_TYPE> ret = new ArrayList<CHILDREN_TYPE>();

    for (CHILDREN_TYPE child : childrenList) {
      if (objectMatcher.matches(child, target)) {
        ret.add(child);
      }
    }

    return ret;
  }

  // --------------------------------------------------------------------------------

  @Override
  public <TARGET> /*  */CHILDREN_TYPE/* */findSingChildBy(
      ObjectMatcher<CHILDREN_TYPE, TARGET> objectMatcher, TARGET target) {

    CHILDREN_TYPE ret = null;

    for (CHILDREN_TYPE child : childrenList) {
      if (objectMatcher.matches(child, target)) {
        if (ret != null) {
          throw new IllegalStateException();
        } else {
          ret = child;
        }
      }
    }

    return ret;
  }

  // --------------------------------------------------------------------------------

  @Override
  public void visitChildren(ObjectVisitor<CHILDREN_TYPE> objectVisitor) {
    for (CHILDREN_TYPE child : childrenList) {
      objectVisitor.visit(child);
    }
  }
}
