package org.cyrano.timetable.ui.model.hour;

import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.List;

import org.cyrano.timetable.ui.model.timetable.Slot;
import org.cyrano.utils.ListUtils;
import org.cyrano.utils.Rect;

public class HourModel {

  private final List<Hour> hourList = //
  new ArrayList<Hour>();

  private final String[] cols;
  private final String[] rows;

  // --------------------------------------------------------------------------------

  private int colW;
  private int rowH;

  // --------------------------------------------------------------------------------

  public HourModel(String[] cols, String[] rows) {
    this.cols = cols;
    this.rows = rows;
  }

  // --------------------------------------------------------------------------------

  public void prepare( //
      int xoffset, int yoffset, int w, int h, List<Slot> slotList) {

    hourList.clear();

    colW = w / (cols.length/**/);
    rowH = h / (rows.length - 1);

    for (int col = 0; col < cols.length/**/; col++) {
      for (int row = 0; row < rows.length - 1; row++) {
        Hour hour = new Hour( //
            cols[col/**/], //
            rows[row/**/], //
            rows[row + 1], //
            col * colW + xoffset, //
            row * rowH + yoffset, //
            colW, //
            rowH //
        );

        hourList.add(hour);
      }
    }

    updateCount(slotList);
  }

  // --------------------------------------------------------------------------------

  private void updateCount(List<Slot> slotList) {
    for (Hour hour : hourList) {
      hour.count = 0;

      for (Slot slot : slotList) {
        if (slot.overlap(hour)) {
          hour.count++;
        }
      }
    }
  }

  // --------------------------------------------------------------------------------

  public int getColW() {
    return colW;
  }

  public int getRowH() {
    return rowH;
  }

  // --------------------------------------------------------------------------------

  public List<Hour> getHourList() {
    return new ArrayList<Hour>(hourList);
  }

  // --------------------------------------------------------------------------------

  public HourBlock getHourBlock(Slot slot) {
    List<Hour> ret = new ArrayList<Hour>();

    for (Hour hour : hourList) {
      if (slot.overlap(hour)) {
        ret.add(hour);
      }
    }

    return new HourBlock(ret);
  }

  // --------------------------------------------------------------------------------

  public HourBlock getHourBlock( //
      Hour preBegHour, Hour preSrcHour, Hour preEndHour, Hour newTgtHour) {

    int preBegIndex = hourList.indexOf(preBegHour);
    int preSrcIndex = hourList.indexOf(preSrcHour);
    int preEndIndex = hourList.indexOf(preEndHour);
    int newTgtIndex = hourList.indexOf(newTgtHour);

    if (preBegIndex > preSrcIndex || preSrcIndex > preEndIndex) {
      throw new IllegalArgumentException( //
          preBegHour + ";" + preSrcIndex + ";" + preEndIndex);
    }

    int beg2src = preSrcIndex - preBegIndex; // > 0
    int src2end = preEndIndex - preSrcIndex; // > 0

    int newBegIndex = newTgtIndex - beg2src;
    int newEndIndex = newTgtIndex + src2end;

    if (newBegIndex < ListUtils.beg(hourList)) {
      return null;
    }

    if (newEndIndex > ListUtils.end(hourList)) {
      return null;
    }

    Hour newBegHour = hourList.get(newBegIndex);
    Hour newEndHour = hourList.get(newEndIndex);

    if (!newBegHour.day.equals(newEndHour.day)) {
      return null;
    }

    return new HourBlock( //
        hourList.subList(newBegIndex, newEndIndex + 1));
  }

  // --------------------------------------------------------------------------------

  public Hour findHour(String day, String beg, String end) {
    for (Hour hour : hourList) {
      if (day.equals(hour.day) && //
          beg.equals(hour.beg) && //
          end.equals(hour.end)) {
        return hour;
      }
    }

    return null;
  }

  // --------------------------------------------------------------------------------

  public Hour getHourAt(int x, int y) {
    for (Hour hour : getHourList()) {
      Rect rect = hour.getRect();
      Rectangle r = new Rectangle( //
          rect.x, rect.y, rect.w, rect.h);

      if (r.contains(x, y)) {
        return hour;
      }
    }

    return null;
  }

  // --------------------------------------------------------------------------------

  public String[] getColsText() {
    return cols;
  }

  // --------------------------------------------------------------------------------

  public String[] getRowsText() {
    String[] ret = new String[rows.length - 1];

    for (int i = 0; i < ret.length; i++) {
      ret[i] = rows[i] + " - " + rows[i + 1];
    }

    return ret;
  }
}
