package org.cyrano.timetable.ui.model.timetable;

import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import javax.swing.event.EventListenerList;

import org.cyrano.timetable.ui.model.base.BaseModel;
import org.cyrano.timetable.ui.model.base.Children;
import org.cyrano.timetable.ui.model.base.ModelChangeSupport;
import org.cyrano.timetable.ui.model.base.ObjectVisitor;
import org.cyrano.timetable.ui.model.base.Parent;
import org.cyrano.timetable.ui.model.base.ParentChildrenImpl;
import org.cyrano.timetable.ui.model.base.Setter;
import org.cyrano.timetable.ui.model.base.StructureChangeSupport;
import org.cyrano.timetable.ui.model.base.VisibilityChangeSupport;
import org.cyrano.timetable.ui.model.hour.HourModel;
import org.cyrano.utils.Rect;

public class Root implements BaseModel {

  private EventListenerList eventListenerList = //
  new EventListenerList();

  // --------------------------------------------------------------------------------

  private VisibilityChangeSupport visibilityChangeSupport = //
  new VisibilityChangeSupport(eventListenerList);

  private StructureChangeSupport structureChangeSupport = //
  new StructureChangeSupport(eventListenerList);

  private ModelChangeSupport modelChangeSupport = //
  new ModelChangeSupport(eventListenerList);

  // --------------------------------------------------------------------------------

  private DisplayMode displayMode = DisplayMode.SEMS;

  private Parent<Root, Sems> semsList;

  private Parent<Root, Prof> profList;

  private Parent<Root, Room> roomList;

  private HourModel hourModel;

  // --------------------------------------------------------------------------------

  public Root(HourModel hourModel) {
    this.hourModel = hourModel;

    ParentChildrenImpl<Root, Sems> semsList = //
    new ParentChildrenImpl<Root, Sems>(this);
    semsList.setSetter(new Setter<Sems, Children<Root>>() {
      @Override
      public void set(Sems target, Children<Root> value) {
        target.setRootRef(value);
      }
    });
    semsList.setComparator(new Comparator<Sems>() {
      @Override
      public int compare(Sems s1, Sems s2) {
        return s1.getDisplayName().compareTo(s2.getDisplayName());
      }
    });
    semsList.setStructureChangeSupport(structureChangeSupport);
    this.semsList = semsList;

    // ----------------------------------------

    ParentChildrenImpl<Root, Prof> profList = //
    new ParentChildrenImpl<Root, Prof>(this);
    profList.setSetter(new Setter<Prof, Children<Root>>() {
      @Override
      public void set(Prof target, Children<Root> value) {
        target.setRootRef(value);
      }
    });
    profList.setComparator(new Comparator<Prof>() {
      @Override
      public int compare(Prof p1, Prof p2) {
        return p1.getDisplayName().compareTo(p2.getDisplayName());
      }
    });
    profList.setStructureChangeSupport(structureChangeSupport);
    this.profList = profList;

    // ----------------------------------------

    ParentChildrenImpl<Root, Room> roomList = //
    new ParentChildrenImpl<Root, Room>(this);
    roomList.setSetter(new Setter<Room, Children<Root>>() {
      @Override
      public void set(Room target, Children<Root> value) {
        target.setRootRef(value);
      }
    });
    roomList.setComparator(new Comparator<Room>() {
      @Override
      public int compare(Room r1, Room r2) {
        return r1.getDisplayName().compareTo(r2.getDisplayName());
      }
    });
    roomList.setStructureChangeSupport(structureChangeSupport);
    this.roomList = roomList;
  }

  // --------------------------------------------------------------------------------

  public VisibilityChangeSupport getVisibilityChangeSupport() {
    return visibilityChangeSupport;
  }

  // --------------------------------------------------------------------------------

  public StructureChangeSupport getStructureChangeSupport() {
    return structureChangeSupport;
  }

  // --------------------------------------------------------------------------------

  public ModelChangeSupport getModelChangeSupport() {
    return modelChangeSupport;
  }

  // --------------------------------------------------------------------------------

  public DisplayMode getDisplayMode() {
    return displayMode;
  }

  public void setDisplayMode(DisplayMode displayMode) {
    this.displayMode = displayMode;
  }

  // --------------------------------------------------------------------------------

  public HourModel getHourModel() {
    return hourModel;
  }

  // --------------------------------------------------------------------------------

  public Parent<Root, Sems> getSemsList() {
    return semsList;
  }

  public Parent<Root, Prof> getProfList() {
    return profList;
  }

  public Parent<Root, Room> getRoomList() {
    return roomList;
  }

  // --------------------------------------------------------------------------------

  public Sems findSemsByName(String name) {
    return semsList.findSingChildBy(Sems.SEMS_BY_NAME_MATCHER, name);
  }

  public Prof findProfByInit(String init) {
    return profList.findSingChildBy(Prof.PROF_BY_INIT_MATCHER, init);
  }

  public Room findRoomByName(String name) {
    return roomList.findSingChildBy(Room.ROOM_BY_NAME_MATCHER, name);
  }

  // --------------------------------------------------------------------------------

  // TODO: collect pattern on Parent
  public Cour findCourByName(String name) {
    for (Sems semester : semsList.getChildrenList()) {
      Cour course = semester.findCourByName(name);

      if (course != null) {
        return course;
      }
    }

    return null;
  }

  // --------------------------------------------------------------------------------

  public Slot getSlotAt(int x, int y) {
    for (Slot slot : getAllChildrenSlots()) {
      Rect rect = slot.getRect();
      Rectangle r = new Rectangle( //
          rect.x, rect.y, rect.w, rect.h);

      if (r.contains(x, y)) {
        return slot;
      }
    }

    return null;
  }

  // --------------------------------------------------------------------------------

  public Rect rectangleFor( //
      String courseName, String sectionId, //
      String day, String beg, String end) {

    Slot slot = findSlot(courseName, sectionId, day, beg, end);

    if (slot == null) {
      return null;
    }

    return slot.getRect();
  }

  // --------------------------------------------------------------------------------

  private Slot findSlot( //
      String name, String numb, //
      String day, String beg, String end) {

    Cour cour = findCourByName(name);

    if (cour == null) {
      return null;
    }

    Sect sect = cour.findSectByNumb(numb);

    if (sect == null) {
      return null;
    }

    return sect.findSlotByDayTime(day, beg, end);
  }

  // --------------------------------------------------------------------------------

  public void clearValidations() {
    semsList.visitChildren(//
        new ObjectVisitor<Sems>() {
          @Override
          public void visit(Sems object) {
            object.clearValidations();
          }
        });
  }

  // --------------------------------------------------------------------------------
  // BaseModel
  // --------------------------------------------------------------------------------

  @Override
  public Root getRoot() {
    return this;
  }

  // --------------------------------------------------------------------------------

  @Override
  public String getDisplayName() {
    return _I18NRoot.rootName();
  }

  // --------------------------------------------------------------------------------

  @Override
  public void setVisible(final boolean visible) {
    switch (displayMode) {
      case SEMS :
        semsList.visitChildren(//
            new ObjectVisitor<Sems>() {
              @Override
              public void visit(Sems object) {
                object.setVisible(visible);
              }
            });
        break;
      case PROF :
        profList.visitChildren(//
            new ObjectVisitor<Prof>() {
              @Override
              public void visit(Prof object) {
                object.setVisible(visible);
              }
            });
        break;
      case ROOM :
        roomList.visitChildren(//
            new ObjectVisitor<Room>() {
              @Override
              public void visit(Room object) {
                object.setVisible(visible);
              }
            });
        break;
      default :
        throw new IllegalStateException( //
            displayMode.toString());
    }
  }

  // --------------------------------------------------------------------------------

  // TODO: collect pattern on Parent
  @Override
  public List<Slot> getAllChildrenSlots() {
    List<Slot> ret = new ArrayList<Slot>();

    for (Sems sems : semsList.getChildrenList()) {
      ret.addAll(sems.getAllChildrenSlots());
    }

    return ret;
  }

  // --------------------------------------------------------------------------------

  @Override
  public void shallowDataCopyFrom(BaseModel baseModel) {
    throw new UnsupportedOperationException();
  }
}
