package org.cyrano.timetable.ui.model.timetable;

import java.util.List;

import org.cyrano.timetable.ui.model.base.BaseModel;
import org.cyrano.timetable.ui.model.base.Children;
import org.cyrano.timetable.ui.model.base.ModelChangeEvent;
import org.cyrano.timetable.ui.model.base.ObjectMatcher;
import org.cyrano.timetable.ui.model.base.Parent;
import org.cyrano.timetable.ui.model.base.ParentChildrenImpl;
import org.cyrano.timetable.ui.model.base.Setter;
import org.cyrano.timetable.ui.model.base.VisibilityChangeEvent;

public class Sect implements BaseModel {

  public static final ObjectMatcher<Sect, String> SECT_BY_NUMB_MATCHER = //
  new ObjectMatcher<Sect, String>() {
    @Override
    public boolean matches(Sect object, String target) {
      return target.equals(object.getNumb());
    }
  };

  // --------------------------------------------------------------------------------

  private Children<Cour> courRef;

  private Parent<Sect, Slot> slotList;

  private String numb;

  private Prof prof;

  private boolean visible = true;

  // --------------------------------------------------------------------------------

  public Sect() {
    ParentChildrenImpl<Sect, Slot> slotList = //
    new ParentChildrenImpl<Sect, Slot>(this);
    slotList.setSetter(new Setter<Slot, Children<Sect>>() {
      @Override
      public void set(Slot target, Children<Sect> value) {
        target.setSectRef(value);
      }
    });
    this.slotList = slotList;
  }

  // --------------------------------------------------------------------------------

  public Children<Cour> getCourRef() {
    return courRef;
  }

  public void setCourRef(Children<Cour> courRef) {
    this.courRef = courRef;

    if (courRef != null) {
      slotList.setStructureChangeSupport( //
          getRoot().getStructureChangeSupport());
    } else {
      slotList.setStructureChangeSupport(null);
    }
  }

  // --------------------------------------------------------------------------------

  public Parent<Sect, Slot> getSlotList() {
    return slotList;
  }

  // --------------------------------------------------------------------------------

  public String getNumb() {
    return numb;
  }

  public void setNumb(String numb) {
    this.numb = numb;
  }

  // --------------------------------------------------------------------------------

  public Prof getProf() {
    return prof;
  }

  public void setProf(Prof prof) {
    this.prof = prof;
  }

  // --------------------------------------------------------------------------------

  public Slot findSlotByDayTime(String day, String beg, String end) {
    for (Slot slot : slotList.getChildrenList()) {
      if (day.equals(slot.getDay()) && //
          beg.equals(slot.getBeg()) && //
          end.equals(slot.getEnd())) {
        return slot;
      }
    }

    return null;
  }

  // --------------------------------------------------------------------------------

  public Slot findSlotByOverlap(String day, String beg, String end) {
    for (Slot slot : slotList.getChildrenList()) {
      if (slot.overlap(day, beg, end)) {
        return slot;
      }
    }

    return null;
  }

  // --------------------------------------------------------------------------------

  public void clearValidations() {
    for (Slot slot : slotList.getChildrenList()) {
      slot.clearValidations();
    }
  }

  // --------------------------------------------------------------------------------

  public boolean getRoomViolation() {
    for (Slot slot : slotList.getChildrenList()) {
      if (slot.getRoomViolation()) {
        return true;
      }
    }

    return false;
  }

  // --------------------------------------------------------------------------------

  public boolean getProfViolation() {
    for (Slot slot : slotList.getChildrenList()) {
      if (slot.getProfViolation()) {
        return true;
      }
    }

    return false;
  }

  // --------------------------------------------------------------------------------

  public boolean getSectViolation() {
    for (Slot slot : slotList.getChildrenList()) {
      if (slot.getSectViolation()) {
        return true;
      }
    }

    return false;
  }

  // --------------------------------------------------------------------------------

  public boolean getSemsViolation() {
    for (Slot slot : slotList.getChildrenList()) {
      if (slot.getSemsViolation()) {
        return true;
      }
    }

    return false;
  }

  // --------------------------------------------------------------------------------
  // BaseModel
  // --------------------------------------------------------------------------------

  @Override
  public Root getRoot() {
    return courRef != null ? courRef.getParentRef().getRoot() : null;
  }

  // --------------------------------------------------------------------------------

  @Override
  public String getDisplayName() {
    return numb;
  }

  // --------------------------------------------------------------------------------

  public boolean getVisible() {
    return visible;
  }

  @Override
  public void setVisible(boolean visible) {
    this.visible = visible;
    getRoot().getVisibilityChangeSupport(). //
        fireVisibilityChange(new VisibilityChangeEvent(this));
  }

  // --------------------------------------------------------------------------------

  @Override
  public List<Slot> getAllChildrenSlots() {
    return slotList.getChildrenList();
  }

  // --------------------------------------------------------------------------------

  @Override
  public void shallowDataCopyFrom(BaseModel baseModel) {
    if (!(baseModel instanceof Sect)) {
      throw new IllegalArgumentException();
    }

    Sect from = (Sect) baseModel;

    this.numb = from.numb;
    this.prof = from.prof;

    if (getRoot() != null) {
      getRoot().getModelChangeSupport().fireModelChange(new ModelChangeEvent(this, this));
    }
  }
}
