package org.cyrano.timetable.ui.model.timetable;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import org.cyrano.timetable.ui.model.base.BaseModel;
import org.cyrano.timetable.ui.model.base.Children;
import org.cyrano.timetable.ui.model.base.ColorProvider;
import org.cyrano.timetable.ui.model.base.ModelChangeEvent;
import org.cyrano.timetable.ui.model.base.ObjectMatcher;
import org.cyrano.timetable.ui.model.base.ObjectVisitor;
import org.cyrano.timetable.ui.model.base.Parent;
import org.cyrano.timetable.ui.model.base.ParentChildrenImpl;
import org.cyrano.timetable.ui.model.base.Setter;

public class Sems implements BaseModel, ColorProvider {

  public static final ObjectMatcher<Sems, String> SEMS_BY_NAME_MATCHER = //
  new ObjectMatcher<Sems, String>() {
    @Override
    public boolean matches(Sems object, String target) {
      return target.equals(object.getName());
    }
  };

  // --------------------------------------------------------------------------------

  private Children<Root> rootRef;

  private Parent<Sems, Cour> courList;

  private String name;

  private Color background;

  // --------------------------------------------------------------------------------

  public Sems() {
    ParentChildrenImpl<Sems, Cour> courList = //
    new ParentChildrenImpl<Sems, Cour>(this);
    courList.setSetter(new Setter<Cour, Children<Sems>>() {
      @Override
      public void set(Cour target, Children<Sems> value) {
        target.setSemsRef(value);
      }
    });
    courList.setComparator(new Comparator<Cour>() {
      @Override
      public int compare(Cour c1, Cour c2) {
        return c1.getDisplayName().compareTo(c2.getDisplayName());
      }
    });
    this.courList = courList;
  }

  // --------------------------------------------------------------------------------

  public Children<Root> getRootRef() {
    return rootRef;
  }

  public void setRootRef(Children<Root> rootRef) {
    this.rootRef = rootRef;

    if (rootRef != null) {
      courList.setStructureChangeSupport( //
          getRoot().getStructureChangeSupport());
    } else {
      courList.setStructureChangeSupport(null);
    }
  }

  // --------------------------------------------------------------------------------

  public Parent<Sems, Cour> getCourList() {
    return courList;
  }

  // --------------------------------------------------------------------------------

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  // --------------------------------------------------------------------------------

  public Cour findCourByName(String name) {
    return courList.findSingChildBy( //
        Cour.COUR_BY_NAME_MATCHER, name);
  }

  // --------------------------------------------------------------------------------

  public void clearValidations() {
    for (Cour cour : courList.getChildrenList()) {
      cour.clearValidations();
    }
  }

  // --------------------------------------------------------------------------------
  // ColorProvider
  // --------------------------------------------------------------------------------

  @Override
  public Color getBackground() {
    return background;
  }

  public void setBackground(Color background) {
    this.background = background;
  }

  // --------------------------------------------------------------------------------
  // BaseModel
  // --------------------------------------------------------------------------------

  @Override
  public Root getRoot() {
    return rootRef != null ? rootRef.getParentRef() : null;
  }

  // --------------------------------------------------------------------------------

  @Override
  public String getDisplayName() {
    return getName();
  }

  // --------------------------------------------------------------------------------

  @Override
  public void setVisible(final boolean visible) {
    courList.visitChildren(//
        new ObjectVisitor<Cour>() {
          @Override
          public void visit(Cour object) {
            object.setVisible(visible);
          }
        });
  }

  // --------------------------------------------------------------------------------

  @Override
  public List<Slot> getAllChildrenSlots() {
    List<Slot> ret = new ArrayList<Slot>();

    for (Cour cour : courList.getChildrenList()) {
      ret.addAll(cour.getAllChildrenSlots());
    }

    return ret;
  }

  // --------------------------------------------------------------------------------

  @Override
  public void shallowDataCopyFrom(BaseModel baseModel) {
    if (!(baseModel instanceof Sems)) {
      throw new IllegalArgumentException();
    }

    Sems from = (Sems) baseModel;

    this.name = from.name;
    this.background = from.background;

    if (getRoot() != null) {
      getRoot().getModelChangeSupport().fireModelChange(new ModelChangeEvent(this, this));
    }
  }
}
