package org.cyrano.timetable.ui.model.timetable;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;

import org.cyrano.timetable.ui.model.base.BaseModel;
import org.cyrano.timetable.ui.model.base.Children;
import org.cyrano.timetable.ui.model.hour.Hour;
import org.cyrano.utils.Rect;

public class Slot implements Comparable<Slot>, BaseModel {

  private final Rect rect = new Rect();

  private Children<Sect> sectRef;

  private String day;
  private String beg;
  private String end;

  private String type;

  private Room room;

  private boolean roomViolation;
  private boolean profViolation;
  private boolean sectViolation;
  private boolean semsViolation;

  // --------------------------------------------------------------------------------

  public Slot() {
    // Empty
  }

  // --------------------------------------------------------------------------------

  public Slot(//
      String day, String beg, String end, //
      Room room, String type) {

    this.day = day;
    this.beg = beg;
    this.end = end;

    this.room = room;
    this.type = type;
  }

  // --------------------------------------------------------------------------------

  public Children<Sect> getSectRef() {
    return sectRef;
  }

  public void setSectRef(Children<Sect> sectRef) {
    this.sectRef = sectRef;
  }

  // --------------------------------------------------------------------------------

  public String getDay() {
    return day;
  }

  public void setDay(String day) {
    this.day = day;
  }

  // --------------------------------------------------------------------------------

  public String getBeg() {
    return beg;
  }

  public void setBeg(String beg) {
    this.beg = beg;
  }

  // --------------------------------------------------------------------------------

  public String getEnd() {
    return end;
  }

  public void setEnd(String end) {
    this.end = end;
  }

  // --------------------------------------------------------------------------------

  public Room getRoom() {
    return room;
  }

  public void setRoom(Room room) {
    this.room = room;
  }

  // --------------------------------------------------------------------------------

  public String getType() {
    return type;
  }

  public void setType(String type) {
    this.type = type;
  }

  // --------------------------------------------------------------------------------

  public Rect getRect() {
    return rect;
  }

  // --------------------------------------------------------------------------------

  public boolean overlap(Slot slot) {
    return overlap(slot.day, slot.beg, slot.end);
  }

  public boolean overlap(Hour hour) {
    return overlap(hour.day, hour.beg, hour.end);
  }

  // --------------------------------------------------------------------------------

  public boolean overlap(String day, String beg, String end) {
    if (!this.day.equals(day)) {
      return false;
    }

    if (this.end.compareTo(/* */beg) <= 0) {
      return false;
    }

    if (/* */end.compareTo(this.beg) <= 0) {
      return false;
    }

    return true;
  }

  // --------------------------------------------------------------------------------

  @Override
  public int compareTo(Slot slot) {
    int dayCmp = this.getDay().compareTo(slot.getDay());

    if (dayCmp != 0) {
      return dayCmp;
    }

    int begCmp = this.getBeg().compareTo(slot.getBeg());

    if (begCmp != 0) {
      return begCmp;
    }

    return this.getEnd().compareTo(slot.getEnd());
  }

  // --------------------------------------------------------------------------------

  public void clearValidations() {
    roomViolation = false;
    profViolation = false;
    sectViolation = false;
    semsViolation = false;
  }

  // --------------------------------------------------------------------------------

  public boolean getRoomViolation() {
    return roomViolation;
  }

  public void setRoomViolation(boolean roomViolation) {
    this.roomViolation = roomViolation;
  }

  // --------------------------------------------------------------------------------

  public boolean getProfViolation() {
    return profViolation;
  }

  public void setProfViolation(boolean profViolation) {
    this.profViolation = profViolation;
  }

  // --------------------------------------------------------------------------------

  public boolean getSectViolation() {
    return sectViolation;
  }

  public void setSectViolation(boolean sectViolation) {
    this.sectViolation = sectViolation;
  }

  // --------------------------------------------------------------------------------

  public boolean getSemsViolation() {
    return semsViolation;
  }

  public void setSemsViolation(boolean semsViolation) {
    this.semsViolation = semsViolation;
  }

  // --------------------------------------------------------------------------------

  public Color getBackground() {
    DisplayMode displayMode = getRoot().getDisplayMode();

    switch (displayMode) {
      case SEMS :
        return sectRef.getParentRef(). //
            getCourRef().getParentRef(). //
            getSemsRef().getParentRef().getBackground();
      case PROF :
        return sectRef.getParentRef(). //
            getProf().getBackground();
      case ROOM :
        return getRoom().getBackground();
      default :
        throw new IllegalStateException(displayMode.toString());
    }
  }

  // --------------------------------------------------------------------------------
  // BaseModel
  // --------------------------------------------------------------------------------

  @Override
  public Root getRoot() {
    return sectRef != null ? sectRef.getParentRef().getRoot() : null;
  }

  // --------------------------------------------------------------------------------

  @Override
  public String getDisplayName() {
    throw new UnsupportedOperationException();
  }

  // --------------------------------------------------------------------------------

  public boolean getVisible() {
    DisplayMode displayMode = getRoot().getDisplayMode();

    switch (displayMode) {
      case SEMS :
        return sectRef.getParentRef().getVisible();
      case PROF :
        return getVisibleByProf();
      case ROOM :
        return getVisibleByRoom();
      default :
        throw new IllegalStateException(displayMode.toString());
    }
  }

  // --------------------------------------------------------------------------------

  private boolean getVisibleByProf() {
    Prof prof = sectRef.getParentRef().getProf();
    return prof != null ? prof.getVisible() : false;
  }

  private boolean getVisibleByRoom() {
    return room != null ? room.getVisible() : false;
  }

  // --------------------------------------------------------------------------------

  @Override
  public void setVisible(boolean visible) {
    throw new UnsupportedOperationException();
  }

  // --------------------------------------------------------------------------------

  @Override
  public List<Slot> getAllChildrenSlots() {
    List<Slot> ret = new ArrayList<Slot>();

    ret.add(this);

    return ret;
  }

  // --------------------------------------------------------------------------------

  @Override
  public void shallowDataCopyFrom(BaseModel baseModel) {
    if (!(baseModel instanceof Slot)) {
      throw new IllegalArgumentException();
    }

    Slot from = (Slot) baseModel;

    this.room = from.room;
  }
}
