package org.cyrano.timetable.ui.reports;

import java.io.File;
import java.io.FileOutputStream;
import java.text.MessageFormat;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellRangeAddress;
import org.cyrano.timetable.ui.model.timetable.Cour;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.timetable.ui.model.timetable.Sect;
import org.cyrano.timetable.ui.model.timetable.Sems;
import org.cyrano.timetable.ui.model.timetable.Slot;

public class SemsReport {

  private static final int HEADER_HEIGHT = 2;

  // --------------------------------------------------------------------------------

  private Workbook workbook;

  private CellStyle dataCellStyle;
  private CellStyle roomCellStyle;
  private CellStyle headCellStyle;

  private Root root;

  // --------------------------------------------------------------------------------

  public SemsReport(Root root) {
    this.root = root;
  }

  // --------------------------------------------------------------------------------

  private int getMergedRowsForSemester(Sems sems) {
    int ret = 0;

    for (Cour cour : sems.getCourList().getChildrenList()) {
      ret += getMergedRowsForCourse(cour);
    }

    return ret;
  }

  // --------------------------------------------------------------------------------

  private int getMergedRowsForCourse(Cour cour) {
    return cour.getSectList().getChildrenList().size();
  }

  // --------------------------------------------------------------------------------

  private void initHeadCellStyle(Workbook wb) {
    headCellStyle = wb.createCellStyle();
    Font f = wb.createFont();
    f.setFontName("Bitstream Vera Sans");
    f.setFontHeightInPoints((short) 10);
    f.setBoldweight(Font.BOLDWEIGHT_BOLD);
    f.setItalic(true);
    headCellStyle.setFillBackgroundColor(IndexedColors.AQUA.getIndex());
    headCellStyle.setFillPattern(CellStyle.BIG_SPOTS);
    headCellStyle.setFillBackgroundColor(IndexedColors.GREEN.getIndex());
    headCellStyle.setFillPattern(CellStyle.BIG_SPOTS);
    headCellStyle.setFont(f);
    headCellStyle.setBorderBottom(CellStyle.BORDER_MEDIUM);
    headCellStyle.setBorderTop(CellStyle.BORDER_MEDIUM);
    headCellStyle.setBorderLeft(CellStyle.BORDER_MEDIUM);
    headCellStyle.setBorderRight(CellStyle.BORDER_MEDIUM);
    headCellStyle.setAlignment(CellStyle.ALIGN_CENTER);
    headCellStyle.setVerticalAlignment(CellStyle.VERTICAL_TOP);
  }

  // --------------------------------------------------------------------------------

  private void initDataCellStyle(Workbook wb) {
    dataCellStyle = wb.createCellStyle();
    Font f = wb.createFont();
    f.setFontName("Bitstream Vera Sans");
    f.setFontHeightInPoints((short) 10);
    f.setBoldweight(Font.BOLDWEIGHT_BOLD);
    dataCellStyle.setFont(f);
    dataCellStyle.setBorderBottom(CellStyle.BORDER_MEDIUM);
    dataCellStyle.setBorderTop(CellStyle.BORDER_MEDIUM);
    dataCellStyle.setBorderLeft(CellStyle.BORDER_MEDIUM);
    dataCellStyle.setBorderRight(CellStyle.BORDER_MEDIUM);
    dataCellStyle.setAlignment(CellStyle.ALIGN_CENTER);
    dataCellStyle.setVerticalAlignment(CellStyle.VERTICAL_TOP);
  }

  // --------------------------------------------------------------------------------

  private void initRoomCellStyle(Workbook wb) {
    roomCellStyle = wb.createCellStyle();
    Font f = wb.createFont();
    f.setFontName("Bitstream Vera Sans Mono");
    f.setFontHeightInPoints((short) 10);
    f.setBoldweight(Font.BOLDWEIGHT_BOLD);
    roomCellStyle.setFont(f);
    roomCellStyle.setBorderBottom(CellStyle.BORDER_MEDIUM);
    roomCellStyle.setBorderTop(CellStyle.BORDER_MEDIUM);
    roomCellStyle.setBorderLeft(CellStyle.BORDER_MEDIUM);
    roomCellStyle.setBorderRight(CellStyle.BORDER_MEDIUM);
    roomCellStyle.setAlignment(CellStyle.ALIGN_RIGHT);
    roomCellStyle.setVerticalAlignment(CellStyle.VERTICAL_TOP);
  }

  // --------------------------------------------------------------------------------

  public void generate() {
    workbook = new HSSFWorkbook();
    Sheet s = workbook.createSheet();
    // declare a row object reference
    Row r = null;
    Cell c = null;

    initDataCellStyle(workbook);
    initRoomCellStyle(workbook);
    initHeadCellStyle(workbook);

    // set the sheet name in Unicode
    workbook.setSheetName(0, "\u0422\u0435\u0441\u0442\u043E\u0432\u0430\u044F "
        + "\u0421\u0442\u0440\u0430\u043D\u0438\u0447\u043A\u0430");
    // in case of plain ascii
    // wb.setSheetName(0, "HSSF Test");
    // create a sheet with 30 rows (0-29)

    //    r = s.createRow(0);
    //    s.addMergedRegion(new CellRangeAddress( //
    //        0, 0, 0, 4));
    //    c = r.createCell(0);
    //    c.setCellValue("Title");

    r = s.createRow(1);
    c = r.createCell(0);
    c.setCellValue("Semester");
    c.setCellStyle(headCellStyle);
    c = r.createCell(1);
    c.setCellValue("Course");
    c.setCellStyle(headCellStyle);
    c = r.createCell(2);
    c.setCellValue("Section");
    c.setCellStyle(headCellStyle);
    c = r.createCell(3);
    c.setCellValue("Professor");
    c.setCellStyle(headCellStyle);
    c = r.createCell(4);
    c.setCellValue("Schedule");
    c.setCellStyle(headCellStyle);

    int rownum = HEADER_HEIGHT;

    for (Sems sems : root.getSemsList().getChildrenList()) {
      int semsMergedRow = getMergedRowsForSemester(sems);

      r = s.createRow(rownum);
      c = r.createCell(0);
      c.setCellValue(sems.getName());
      c.setCellStyle(dataCellStyle);

      for (int i = 1; i < semsMergedRow; i++) {
        r = s.createRow(rownum + i);
        c = r.createCell(0);
        c.setCellStyle(dataCellStyle);
      }

      s.addMergedRegion(new CellRangeAddress( //
          rownum, rownum + semsMergedRow - 1, 0, 0));

      rownum += semsMergedRow;
    }

    rownum = HEADER_HEIGHT;

    for (Sems sems : root.getSemsList().getChildrenList()) {
      for (Cour cour : sems.getCourList().getChildrenList()) {
        int courMergedRow = getMergedRowsForCourse(cour);

        r = s.getRow(rownum);
        c = r.createCell(1);
        c.setCellValue(cour.getName());
        c.setCellStyle(dataCellStyle);

        s.addMergedRegion(new CellRangeAddress( //
            rownum, rownum + courMergedRow - 1, 1, 1));

        rownum += courMergedRow;
      }
    }

    rownum = HEADER_HEIGHT;

    for (Sems sems : root.getSemsList().getChildrenList()) {
      for (Cour cour : sems.getCourList().getChildrenList()) {
        for (Sect sect : cour.getSectList().getChildrenList()) {

          r = s.getRow(rownum);
          c = r.createCell(2);

          c.setCellValue(sect.getNumb());
          c.setCellStyle(dataCellStyle);
          c = r.createCell(3);
          c.setCellValue(sect.getProf().getInit());
          c.setCellStyle(dataCellStyle);

          String string = null;

          for (Slot slot : sect.getSlotList().getChildrenList()) {
            if (string != null) {
              string += "\n";
            } else {
              string = "";
            }

            String format = "{0} {1}-{2} ({3})";

            String slotString = MessageFormat.format(format, //
                slot.getDay(), slot.getBeg(), slot.getEnd(), //
                slot.getRoom().getName());

            string += slotString;

          }

          c = r.createCell(4);
          c.setCellValue(string);
          c.setCellStyle(roomCellStyle);

          r.setHeightInPoints( //
          sect.getSlotList().getChildrenList().size() * s.getDefaultRowHeightInPoints());

          rownum++;
        }
      }
    }

    //    r = s.createRow(rownum);
    //    s.addMergedRegion(new CellRangeAddress( //
    //        rownum, rownum, 0, 4));
    //    c = r.createCell(0);
    //    c.setCellValue("Footer");

    for (int i = 0; i < 4; i++) {
      s.autoSizeColumn((short) i);
    }
  }

  // --------------------------------------------------------------------------------

  public void save(File file) throws Exception {
    FileOutputStream fos = new FileOutputStream(file);
    workbook.write(fos);
    fos.close();
  }
}
