package org.cyrano.timetable.xjc;

import java.io.InputStream;
import java.io.OutputStream;

import javax.xml.XMLConstants;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.PropertyException;
import javax.xml.bind.Unmarshaller;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

/**
 * @author Demián Gutierrez
 */
public class TimeTableXmlLoader {

  private static final String COM_SUN_XML_BIND_OBJECT_FACTORY = //
  "com.sun.xml.bind.ObjectFactory";
  private static final String COM_SUN_XML_INTERNAL_BIND_OBJECT_FACTORY = //
  "com.sun.xml.internal.bind.ObjectFactory";

  private static final String TIMETABLE_XSD = //
  "org/cyrano/timetable/xsd/timetable.xsd";

  // --------------------------------------------------------------------------------
  // Load/Save a Timetable
  // --------------------------------------------------------------------------------

  public Root load(InputStream is) throws Exception {
    JAXBContext jc = JAXBContext.newInstance( //
        "org.cyrano.timetable.xjc.generated");

    Unmarshaller unmarshaller = jc.createUnmarshaller();
    unmarshaller.setSchema(createSchema());
    setObjectFactory(unmarshaller);

    Root xmlTimetable = (Root) unmarshaller.unmarshal(is);

    return xmlTimetable;
  }

  // --------------------------------------------------------------------------------

  public void save(OutputStream os, Root timeTable) throws Exception {
    JAXBContext jc = JAXBContext.newInstance( //
        "org.cyrano.timetable.xjc.generated");

    Marshaller marshaller = jc.createMarshaller();
    marshaller.setSchema(createSchema());

    marshaller.marshal(timeTable, os);
  }

  // --------------------------------------------------------------------------------

  private Schema createSchema() throws Exception {
    SchemaFactory schemaFactory = //
    SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);

    return schemaFactory.newSchema( //
        ClassLoader.getSystemResource(TIMETABLE_XSD));
  }

  // --------------------------------------------------------------------------------
  // Check http://java.net/projects/jaxb/lists/users/archive/2008-07/message/75
  // --------------------------------------------------------------------------------

  private void setObjectFactory(Unmarshaller unmarshaller) //
      throws PropertyException {

    try {
      unmarshaller.setProperty(COM_SUN_XML_BIND_OBJECT_FACTORY, //
          new ObjectFactory());
    } catch (PropertyException e) {
      unmarshaller.setProperty(COM_SUN_XML_INTERNAL_BIND_OBJECT_FACTORY, //
          new ObjectFactory());
    }
  }
}
